"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.prepareExecutionEnv = prepareExecutionEnv;
exports.getNodeBinDir = getNodeBinDir;
exports.getNodeVersionsBaseDir = getNodeVersionsBaseDir;
exports.getNodeDir = getNodeDir;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const util_1 = __importDefault(require("util"));
const env_system_node_version_1 = require("@pnpm/env.system-node-version");
const fetch_1 = require("@pnpm/fetch");
const logger_1 = require("@pnpm/logger");
const node_fetcher_1 = require("@pnpm/node.fetcher");
const store_path_1 = require("@pnpm/store-path");
const load_json_file_1 = __importDefault(require("load-json-file"));
const write_json_file_1 = __importDefault(require("write-json-file"));
const getNodeMirror_1 = require("./getNodeMirror");
const parseNodeSpecifier_1 = require("./parseNodeSpecifier");
const nodeFetchPromises = {};
async function prepareExecutionEnv(config, { extraBinPaths, executionEnv }) {
    if (!executionEnv?.nodeVersion || `v${executionEnv.nodeVersion}` === (0, env_system_node_version_1.getSystemNodeVersion)()) {
        return { extraBinPaths: extraBinPaths ?? [] };
    }
    let nodePathPromise = nodeFetchPromises[executionEnv.nodeVersion];
    if (!nodePathPromise) {
        nodePathPromise = getNodeBinDir({
            ...config,
            useNodeVersion: executionEnv.nodeVersion,
        });
        nodeFetchPromises[executionEnv.nodeVersion] = nodePathPromise;
    }
    return {
        extraBinPaths: [await nodePathPromise, ...extraBinPaths ?? []],
    };
}
async function getNodeBinDir(opts) {
    const fetch = (0, fetch_1.createFetchFromRegistry)(opts);
    const nodesDir = getNodeVersionsBaseDir(opts.pnpmHomeDir);
    const manifestNodeVersion = (await readNodeVersionsManifest(nodesDir))?.default;
    let wantedNodeVersion = opts.useNodeVersion ?? manifestNodeVersion;
    if (opts.useNodeVersion != null) {
        // If the user has specified an invalid version via use-node-version, we should not throw an error. Or else, it will break all the commands.
        // Instead, we should fallback to the manifest node version
        if (!(0, parseNodeSpecifier_1.isValidVersion)(opts.useNodeVersion)) {
            (0, logger_1.globalWarn)(`"${opts.useNodeVersion}" is not a valid Node.js version.`);
            wantedNodeVersion = manifestNodeVersion;
        }
    }
    if (wantedNodeVersion == null) {
        const response = await fetch('https://registry.npmjs.org/node');
        wantedNodeVersion = (await response.json())['dist-tags'].lts; // eslint-disable-line
        if (wantedNodeVersion == null) {
            throw new Error('Could not resolve LTS version of Node.js');
        }
        await (0, write_json_file_1.default)(path_1.default.join(nodesDir, 'versions.json'), {
            default: wantedNodeVersion,
        });
    }
    const { useNodeVersion, releaseChannel } = (0, parseNodeSpecifier_1.parseNodeSpecifier)(wantedNodeVersion);
    const nodeMirrorBaseUrl = (0, getNodeMirror_1.getNodeMirror)(opts.rawConfig, releaseChannel);
    const nodeDir = await getNodeDir(fetch, {
        ...opts,
        useNodeVersion,
        nodeMirrorBaseUrl,
    });
    return process.platform === 'win32' ? nodeDir : path_1.default.join(nodeDir, 'bin');
}
function getNodeVersionsBaseDir(pnpmHomeDir) {
    return path_1.default.join(pnpmHomeDir, 'nodejs');
}
async function getNodeDir(fetch, opts) {
    const nodesDir = getNodeVersionsBaseDir(opts.pnpmHomeDir);
    await fs_1.default.promises.mkdir(nodesDir, { recursive: true });
    const versionDir = path_1.default.join(nodesDir, opts.useNodeVersion);
    if (!fs_1.default.existsSync(versionDir)) {
        const storeDir = await (0, store_path_1.getStorePath)({
            pkgRoot: process.cwd(),
            storePath: opts.storeDir,
            pnpmHomeDir: opts.pnpmHomeDir,
        });
        (0, logger_1.globalInfo)(`Fetching Node.js ${opts.useNodeVersion} ...`);
        await (0, node_fetcher_1.fetchNode)(fetch, opts.useNodeVersion, versionDir, {
            ...opts,
            storeDir,
            retry: {
                maxTimeout: opts.fetchRetryMaxtimeout,
                minTimeout: opts.fetchRetryMintimeout,
                retries: opts.fetchRetries,
                factor: opts.fetchRetryFactor,
            },
        });
    }
    return versionDir;
}
async function readNodeVersionsManifest(nodesDir) {
    try {
        return await (0, load_json_file_1.default)(path_1.default.join(nodesDir, 'versions.json'));
    }
    catch (err) {
        if (util_1.default.types.isNativeError(err) && 'code' in err && err.code === 'ENOENT') {
            return {};
        }
        throw err;
    }
}
//# sourceMappingURL=node.js.map